#include <avr/io.h>
#include "MMC_SD/MMC_SD.h"
#include "FAT/FAT.h"
#include <util/delay.h>
#include "types.h"

// control line pins
#define	LCD_CD	PC7   
#define	LCD_WR 	PC6
#define	LCD_RD  PC5
#define	LCD_CS	PC4
#define	LCD_RES	PC2

// macros for control lines
#define CLR_CD PORTC &= ~(1<<LCD_CD);
#define SET_CD PORTC |=  (1<<LCD_CD);

#define CLR_CS PORTC &= ~(1<<LCD_CS);
#define SET_CS PORTC |=  (1<<LCD_CS);

#define CLR_RESET PORTC &= ~(1<<LCD_RES);
#define SET_RESET PORTC |=  (1<<LCD_RES);

#define CLR_WR PORTC &= ~(1 << LCD_WR);
#define SET_WR PORTC |=  (1 << LCD_WR);
#define CLR_RD PORTC &= ~(1 << LCD_RD);
#define SET_RD PORTC |=  (1 << LCD_RD);

//Buttons and LEDs
#define SW_UP		PIND >= 0xF0
#define LED1_OFF	PORTD &= ~(1 << 1);
#define LED1_ON		PORTD |=  (1 << 1);
#define LED2_OFF	PORTD &= ~(1 << 2);
#define LED2_ON		PORTD |=  (1 << 2);

// color definitions
#define	BLACK	  0x0000
#define	BLUE	  0x001F
#define	RED 	  0xF800
#define	GREEN     0x07E0
#define CYAN	  0x07FF
#define MAGENTA   0xF81F
#define YELLOW    0xFFE0
#define WHITE	  0xFFFF

#define uint8 unsigned char
#define uint16 unsigned short
#define uint32 unsigned long

unsigned char * PATH = "\\batch";

extern uint16 SectorsPerClust;
extern uint16 FirstDataSector; 
extern uint8 FAT32_Enable;

struct FileInfoStruct FileInfo;

struct direntry PictureInfo;
uint16 totalfiles = 0;

//Function Declarations
void delay(unsigned int t);
void writeCommand(uint8 command);
void writeData(uint8 data);
void displayCheckerboard(void);
void displayColorBars();
void displayHome();
void displayInit();
void displayTest(char color);
void slideShow();
/*************************************************/
void delay(unsigned int t)
{
	unsigned char t1;
	while(t--)
	{
		for(t1=11;t1>0;t1--)
		{
			asm("nop");
		}
	}
}
/*************************************************/
void writeCommand(uint8 command)
{
	CLR_CD;   				// command mode
	CLR_CS

	SPI_WriteByte(command);	// set up data on bus

	SET_CS
}

/*************************************************/
void writeData(uint8 data)
{
	SET_CD;					// data mode
	CLR_CS

	SPI_WriteByte(data);	// set up data on bus

	SET_CS
}
/*************************************************/
void displayInit()
{
	int pwm, delta;
	CLR_CS

	writeCommand(0xfd);	// command lock
	writeCommand(0x12);

	writeCommand(0xae);	// display off

	writeCommand(0xa4); 	// Normal Display mode

	writeCommand(0x15);	//set column address
	writeCommand(0x00);	//column address start 00
	writeCommand(0x5f);	//column address end 95

	writeCommand(0x75);	//set row address
	writeCommand(0x00);	//row address start 00
	writeCommand(0x3f);	//row address end 63	

	writeCommand(0x87);	//master current control
	writeCommand(0x0E);	//9/16(160uA)

	writeCommand(0x83);	//Set Contrast for Color R
	writeCommand(0xF0);

	writeCommand(0x82);	//Set Contrast for Color G
	writeCommand(0x55);

	writeCommand(0x81);	//Set Contrast for Color B
	writeCommand(0x65);

	writeCommand(0x8a);
	writeCommand(0x61);
	writeCommand(0x8b);
	writeCommand(0x62);
	writeCommand(0x8c);
	writeCommand(0x63);

	writeCommand(0xa0);	//set re-map & data format
	writeCommand(0x60);									//Flex on Top
//	writeCommand(0x72);									//Flex on Bottom
//	writeCommand(0x70);									//B1 (Flex on Top)

	writeCommand(0xa1);	//set display start line
	writeCommand(0x00);	//start 00 line

	writeCommand(0xa2);	//set display offset
	writeCommand(0x00);

	writeCommand(0xa8);	//set multiplex ratio
	writeCommand(0x3f);	//64MUX							//B2 and B1

	writeCommand(0xad);
	writeCommand(0x8f);									//B2
//	writeCommand(0x8A);									//B1

	writeCommand(0xb0);	//set power save
	writeCommand(0x1a);	//								//B2
//	writeCommand(0x00);      //0x00: not power saving mode	//B1

	writeCommand(0xb1);
	writeCommand(0xf1);	// Phase 2 period Phase 1 period	//B2 and B1

	writeCommand(0xb3);	// Set Display Clock Divide Ratio/ Oscillator Frequency
	writeCommand(0xd0);	// 0.97MHZ 0xd0					//B2
//	writeCommand(0xF0);									//B1

	writeCommand(0xbb);	// set pre-charge
	writeCommand(0x3e);	// 								//B2
//	writeCommand(0x7F);									//B1

//	writeCommand(0xBC);		//set vpb					//B1 only
//	writeCommand(0x7F);

//	writeCommand(0xBD);   //set vpc						//B1 only
//	writeCommand(0x7F);
	
	writeCommand(0xbe);	//set Vcomh
	writeCommand(0x3e);	//0.83Vref						//B2
//	writeCommand(0x7F); 									//B1

	writeCommand(0xB9);

	writeCommand(0xB8); //SET Gray Scale Table
	pwm = 1;
	//The gray scale table setting below is for reference only
	delta=1;
	pwm=pwm; writeCommand(pwm);//pw1
	pwm=pwm+delta; writeCommand(pwm);//pw3
	pwm=pwm+delta; writeCommand(pwm);//pw5
	pwm=pwm+delta; writeCommand(pwm);//pw7
	delta=2;
	pwm=pwm+delta; writeCommand(pwm);//pw9
	pwm=pwm+delta; writeCommand(pwm);//pw11
	pwm=pwm+delta; writeCommand(pwm);//pw13
	
	pwm=pwm+delta; writeCommand(pwm);//pw15
	delta=2;
	pwm=pwm+delta; writeCommand(pwm);//pw17
	pwm=pwm+delta; writeCommand(pwm);//pw19
	pwm=pwm+delta; writeCommand(pwm);//pw21
	pwm=pwm+delta; writeCommand(pwm);//pw23
	delta=3;
	pwm=pwm+delta; writeCommand(pwm);//pw25
	pwm=pwm+delta; writeCommand(pwm);//pw27
	pwm=pwm+delta; writeCommand(pwm);//pw29
	pwm=pwm+delta; writeCommand(pwm);//pw31
	delta=4;
	pwm=pwm+delta; writeCommand(pwm);//pw33
	pwm=pwm+delta; writeCommand(pwm);//pw35
	pwm=pwm+delta; writeCommand(pwm);//pw37
	pwm=pwm+delta; writeCommand(pwm);//pw39
	delta=5;
	pwm=pwm+delta; writeCommand(pwm);//pw41
	pwm=pwm+delta; writeCommand(pwm);//pw43
	pwm=pwm+delta; writeCommand(pwm);//pw45
	pwm=pwm+delta; writeCommand(pwm);//pw47
	delta=6;
	pwm=pwm+delta; writeCommand(pwm);//pw49
	pwm=pwm+delta; writeCommand(pwm);//pw51
	pwm=pwm+delta; writeCommand(pwm);//pw53
	pwm=pwm+delta; writeCommand(pwm);//pw55
	delta=7;
	pwm=pwm+delta; writeCommand(pwm);//pw57
	pwm=pwm+delta; writeCommand(pwm);//pw59
	pwm=pwm+delta; writeCommand(pwm);//pw61
	pwm=pwm+delta; writeCommand(pwm);//pw63

	writeCommand(0xad);	//Select external VCC supply at Display ON
	writeCommand(0x8e); 	//Select External VP voltage supply

	writeCommand(0xaf);	 //display on
}
/*************************************************/
void Display_Home()
{
	writeCommand(0x15);    //  00010101:  Set Column address
	writeCommand(0x00);    //  00000000:  A[6:0] sets the column start address from 0-95, RESET=00d.
	writeCommand(0x5F);    //  01011111:  B[6:0] sets the column end address from 0-95 RESET=95d.

	writeCommand(0x75);    //  10000001:  Set Row Address
	writeCommand(0x00);    //  00000000:  A[5:0] sets the row start address from 0-63, RESET=00d.
	writeCommand(0x3F);    //  00111111:  B[5:0] sets the row end address from 0-63, RESET=63d.
}
/*************************************************/
//void display_rgb(unsigned int data)
void displayColorBars()
{
//BLACK	  0x0000 84-96
//BLUE	  0x001F 73-84
//RED 	  0xF800 61-72
//GREEN   0x07E0 49-60
//CYAN	  0x07FF 37-48
//MAGENTA 0xF81F 25-36
//YELLOW  0xFFE0 13-24
//WHITE	  0xFFFF 0 -12

	Display_Home();

// white, red, green, blue screen
	int i,j;
	for(i=0;i<64;i++)
	{
		for(j=0;j<96;j++)
		{
			if(j > 84)
			{
				writeData(BLACK>>8);
				writeData(BLACK&0x00FF);
			}
			else if(j > 72)
			{
				writeData(BLUE>>8);
				writeData(BLUE&0x00FF);
			}
			else if(j > 60)
			{
				writeData(RED>>8);
				writeData(RED&0x00FF);
			}
			else if(j > 48)
			{
				writeData(GREEN>>8);
				writeData(GREEN&0x00FF);
			}
			else if(j > 36)
			{
				writeData(CYAN>>8);
				writeData(CYAN&0x00FF);
			}
			else if(j > 24)
			{
				writeData(MAGENTA>>8);
				writeData(MAGENTA&0x00FF);
			}
			else if(j > 12)
			{
				writeData(YELLOW>>8);
				writeData(YELLOW&0x00FF);
			}
			else
			{
				writeData(WHITE>>8);
				writeData(WHITE&0x00FF);
			}
		}
	}
}
/*************************************************/
void Picture_Slide_Show()
{
	uint8 i;              //loop variable
	uint16 j;             //loop variable
	uint32 p;             //cluster
	uint16 *buffer;				//buffer
	uint16 pics = 1;
	uint16 slide_show_flag = 0;
	uint8 sector;
	uint32 pixels;

	if (totalfiles == 0)
	return;

	buffer = malloc(512);
	if (buffer == 0)
	return;

next: 	//label for "goto"

	Search(PATH, &PictureInfo, &pics);                                         //find the file
	p = PictureInfo.deStartCluster + (((uint32)PictureInfo.deHighClust) << 16);//the first cluster of the file

	sector = 0;

	FAT_LoadPartCluster(p, sector, buffer);	//read a sector

	pixels = (uint32) 96 * 64 + 1; // total # of pixels to write

	Display_Home();
	j = 0; // byte count
	
	while (1)
	{
		writeData(buffer[j]);  // write 8 bits
		writeData(buffer[j]>>8);  // write 8 bits
		pixels--;               // which is one pixel
		j++;                    // word count

		if (pixels == 0)        
		break;

		if (j == 256)           // time for a new sector     
		{
			sector++;
			if (sector == SectorsPerClust)
			{
				p = FAT_NextCluster(p);				// read next cluster
				sector = 0;
			}

			FAT_LoadPartCluster(p, sector, buffer);	// read a sector
			j = 0;
		}
	}

	if(slide_show_flag)
	{
		for (i=0;i<15;i++)	// delay for a while
		delay(0xffff);

		pics++;					// increment picture number
		if (pics > totalfiles)	// if last
		pics = 1;			// wrap around

		goto next;				// go show next pic
	}
	free(buffer);
}
/*************************************************/
void LCD_test(char color)
{
	int i,j;

	writeCommand(0x15);    //  00010101:  Set Column address
	writeCommand(0x00);    //  00000000:  A[6:0] sets the column start address from 0-95, RESET=00d.
	writeCommand(0x5F);    //  01011111:  B[6:0] sets the column end address from 0-95 RESET=95d.

	writeCommand(0x75);    //  10000001:  Set Row Address
	writeCommand(0x00);    //  00000000:  A[5:0] sets the row start address from 0-63, RESET=00d.
	writeCommand(0x3F);    //  00111111:  B[5:0] sets the row end address from 0-63, RESET=63d.

	for(i=0;i<64;i++)
	{
		for(j=0;j<=96;j++)
		{
			if(color==0)
			{
				writeData(0xff);		//white display
				writeData(0xff);
			}
			if(color==1)
			{
				writeData(0x00);		//blue display
				writeData(0x1f);
			}
			if(color==2)
			{
				writeData(0x07);		//green display
				writeData(0xe0);
			}
			if(color==3)
			{
				writeData(0xf8);		//red display
				writeData(0x00);
			}		
			if(color==4)
			{
				writeData(0x00);		//black display
				writeData(0x00);
			}
		}       
	}
}
/*************************************************/
void displayCheckerboard(void)
{
	int i,j;

	writeCommand(0x15);    //  00010101:  Set Column address
	writeCommand(0x00);    //  00000000:  A[6:0] sets the column start address from 0-95, RESET=00d.
	writeCommand(0x5F);    //  01011111:  B[6:0] sets the column end address from 0-95 RESET=95d.

	writeCommand(0x75);    //  10000001:  Set Row Address
	writeCommand(0x00);    //  00000000:  A[5:0] sets the row start address from 0-63, RESET=00d.
	writeCommand(0x3F);    //  00111111:  B[5:0] sets the row end address from 0-63, RESET=63d.

	unsigned int color1;
	unsigned int color2;
	unsigned int colorTemp;

	color1 = 0xFF;
	color2 = 0x00;

	for(i=0;i<64;i++)
	{
		for(j=0;j<=47;j++)
		{
			//white pixel
			writeData(color1);
			writeData(color1);
			//black display
			writeData(color2);
			writeData(color2);
		}
		colorTemp = color1;
		color1 = color2;
		color2 = colorTemp;
	}
}
/*************************************************/
#define screenDelay (0xFFFF)
int main()
{
	// Give it some initial values
	DDRC  = 0xFE;
 	PORTC = 0xCE;

	uint8 retry = 0;

	OSCCAL = 0xFF;    

	delay(0xFFFF);

	MMC_SD_Init();    //SPI initialize

	sd_restart: 	//label for "goto"

	while (MMC_SD_Reset())  // Initialize SD Card
	{
		retry++;
		if (retry > 254)
		{
			while (1)
			{
				goto sd_restart;				// try again
			}
		}
	}

	delay(0xFFFF);      

	if (FAT_Init())   	//initialize file system - FAT16 and FAT32 are supported
	{
		while (1)
		{
			goto sd_restart;				// try again
		}
	}

	CLR_RESET
	delay(1);
	SET_RESET

	displayInit();
	delay(0x000F);

	displayColorBars();
	delay(screenDelay);

	LCD_test(0);	//White
	delay(screenDelay);

	LCD_test(1);	//Red
	delay(screenDelay);

	LCD_test(2);	//Green
	delay(screenDelay);

	LCD_test(3);	//Blue
	delay(screenDelay);

	displayCheckerboard();
	delay(screenDelay);

	while(1)
	{
		Search(PATH, &PictureInfo, &totalfiles);
		Picture_Slide_Show();
		delay(0xFFFF);
	}

	return 0;

}
/*************************************************/


